// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 51954 $
// $Date: 2022-10-25 08:44:06 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTO_CAL_START			D7		// BCR
#define	AUTO_CAL_PASS			D12		// BCR
#define	CHAN_READY				D13		// BCR
#define	INIT_START				D15		// BCR

#define	IRQ_ACTIVE				0x0800	// BCR
#define	IRQ_INIT_DONE			0x0000	// BCR
#define	IRQ_AUTO_CAL_DONE		0x0100	// BCR
#define	IRQ_MASK				0x0700	// BCR



// prototypes *****************************************************************

static	int		_wait_for_ready(dev_data_t* dev, long ms_limit);



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	switch (arg[0])
	{
		default:						arg[0]	= SDI_IOCTL_QUERY_ERROR;			break;
		case SDI_QUERY_AUTO_CAL_MS:		arg[0]	= dev->cache.auto_cal_ms;			break;
		case SDI_QUERY_CHANNEL_GPS:		arg[0]	= dev->cache.channel_groups;		break;
		case SDI_QUERY_CHANNEL_D16:		arg[0]	= dev->cache.brr_chan_d16;			break;
		case SDI_QUERY_CHANNEL_D17:		arg[0]	= dev->cache.brr_chan_d17;			break;
		case SDI_QUERY_CHANNEL_MAX:		arg[0]	= dev->cache.channels_max;			break;
		case SDI_QUERY_CHANNEL_QTY:		arg[0]	= dev->cache.channel_qty;			break;
		case SDI_QUERY_COUNT:			arg[0]	= SDI_IOCTL_QUERY_LAST + 1;			break;
		case SDI_QUERY_DEVICE_TYPE:		arg[0]	= dev->board_type;					break;
		case SDI_QUERY_DMDMA:			arg[0]	= dev->cache.dmdma;					break;
		case SDI_QUERY_FGEN_MAX:		arg[0]	= dev->cache.rate_gen_fgen_max;		break;
		case SDI_QUERY_FGEN_MIN:		arg[0]	= dev->cache.rate_gen_fgen_min;		break;
		case SDI_QUERY_FIFO_SIZE:		arg[0]	= dev->cache.fifo_size;				break;
		case SDI_QUERY_FORM_FACTOR:		arg[0]	= dev->cache.form_factor;			break;
		case SDI_QUERY_FSAMP_MAX:		arg[0]	= dev->cache.fsamp_max;				break;
		case SDI_QUERY_FSAMP_MIN:		arg[0]	= dev->cache.fsamp_min;				break;
		case SDI_QUERY_INIT_MS:			arg[0]	= dev->cache.initialize_ms;			break;
		case SDI_QUERY_IRQ_MASK:		arg[0]	= dev->cache.irq_mask;				break;
		case SDI_QUERY_RATE_GEN_QTY:	arg[0]	= dev->cache.rate_gen_qty;			break;
		case SDI_QUERY_THRESHOLD_MASK:	arg[0]	= dev->cache.threshold_mask;		break;
		case SDI_QUERY_NRATE_MASK:		arg[0]	= dev->cache.rate_gen_nrate_mask;	break;
		case SDI_QUERY_NRATE_MAX:		arg[0]	= dev->cache.rate_gen_nrate_max;	break;
		case SDI_QUERY_NRATE_MIN:		arg[0]	= dev->cache.rate_gen_nrate_min;	break;
		case SDI_QUERY_NDIV_MASK:		arg[0]	= dev->cache.rate_div_ndiv_mask;	break;
		case SDI_QUERY_NDIV_MAX:		arg[0]	= dev->cache.rate_div_ndiv_max;		break;
		case SDI_QUERY_NDIV_MIN:		arg[0]	= dev->cache.rate_div_ndiv_min;		break;
	}

	return(0);
}



//*****************************************************************************
static int _init_start(dev_data_t* dev, void* arg)
{
	// Initiate initialization.
	os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bcr_32, INIT_START);
	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	u32			bcr;
	int			i;
	int			mask;
	long		ms			= dev->cache.initialize_ms + 5000;
	long		ms_total	= ms;
	os_sem_t	sem;
	int			ret			= 0;
	int			tmp;
	int			value;
	gsc_wait_t	wt;

	if (dev->irq.opened)
	{
		ms_total	*= 2;

		// Safely select the Initialize Done interrupt.
		mask	= IRQ_MASK | IRQ_ACTIVE;
		value	= IRQ_INIT_DONE;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= SDI_WAIT_GSC_INIT_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _init_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: INITILIZE DONE IRQ TIMED OUT AFTER %ld ms.\n",
					dev->model,
					(long) ms);
		}
	}

	// Manually wait for completion as the IRQ wait may have ended early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(dev_data_t* dev, void* arg)
{
	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, AUTO_CAL_START);
	return(0);
}



//*****************************************************************************
static int _auto_calibrate(dev_data_t* dev, void* arg)
{
	u32			bcr;
	u32			mask;
	long		ms		= dev->cache.auto_cal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Auto-Cal Done interrupt.
	mask	= IRQ_MASK | IRQ_ACTIVE;
	value	= IRQ_AUTO_CAL_DONE;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= SDI_WAIT_GSC_AUTO_CAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bcr_32, AUTO_CAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & AUTO_CAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if ((reg & AUTO_CAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_status(dev_data_t* dev, s32* arg)
{
	u32	bcr;

	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & D7)
		arg[0]	= SDI_AUTO_CAL_STATUS_ACTIVE;
	else if (bcr & D12)
		arg[0]	= SDI_AUTO_CAL_STATUS_PASS;
	else
		arg[0]	= SDI_AUTO_CAL_STATUS_FAIL;

	return(0);
}



//*****************************************************************************
static int _ain_buf_clear(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_btr_32, D19, D19);
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_btr_32, 0, D19);
	return(0);
}



//*****************************************************************************
static int _ain_buf_input(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	18

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_btr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			arg[0]	= arg[0]
					? SDI_AIN_BUF_INPUT_DISABLE
					: SDI_AIN_BUF_INPUT_ENABLE;
			break;

		case SDI_AIN_BUF_INPUT_DISABLE:
		case SDI_AIN_BUF_INPUT_ENABLE:

			v	= (arg[0] ? 0 : 1) << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_btr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _ain_buf_thresh(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	dev->cache.threshold_mask
	#define	SHIFT	0

	int	ret;
	u32	v;

	if (arg[0] == -1)
	{
		ret		= 0;
		arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_btr_32);
		arg[0]	&= MASK;
		arg[0]	>>= SHIFT;
	}
	else if ((arg[0] >= 0) && (arg[0] <= dev->cache.threshold_mask))
	{
		v	= arg[0] << SHIFT;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_btr_32, v, MASK);
		ret	= _wait_for_ready(dev, 1000);
	}
	else
	{
		ret	= -EINVAL;
	}

	return(ret);
}



//*****************************************************************************
static int _ain_mode(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3L << SHIFT)
	#define	SHIFT	0

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_AIN_MODE_DIFF:
		case SDI_AIN_MODE_SE:
		case SDI_AIN_MODE_ZERO:
		case SDI_AIN_MODE_VREF:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _ain_range(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3L << SHIFT)
	#define	SHIFT	2

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_AIN_RANGE_1_25V:
		case SDI_AIN_RANGE_2_5V:
		case SDI_AIN_RANGE_5V:
		case SDI_AIN_RANGE_10V:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _ch_grp_src(
	dev_data_t*	dev,
	s32*		arg,
	int			group,
	u32			mask,
	u32			shift)
{
	#undef	MASK
	#undef	SHIFT

	int	ret;
	u32	v;

	if (group >= dev->cache.channel_groups)
	{
		ret	= -EINVAL;
	}
	else
	{
		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case -1:

				ret		= 0;
				arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_rar_32);
				arg[0]	&= mask;
				arg[0]	>>= shift;

				if (arg[0] > SDI_CH_GRP_SRC_DISABLE)
					arg[0]	= SDI_CH_GRP_SRC_DISABLE;

				break;

			case SDI_CH_GRP_SRC_GEN_C:
			case SDI_CH_GRP_SRC_GEN_D:

				if (dev->cache.rate_gen_qty <= 2)
				{
					ret	= -EINVAL;
					break;
				}

				FALLTHROUGH;	/* fall through */

			case SDI_CH_GRP_SRC_GEN_A:
			case SDI_CH_GRP_SRC_GEN_B:
			case SDI_CH_GRP_SRC_EXTERN:
			case SDI_CH_GRP_SRC_DISABLE:

				v	= arg[0] << shift;
				os_reg_mem_mx_u32(dev, dev->vaddr.gsc_rar_32, v, mask);
				ret	= _wait_for_ready(dev, 1000);
				break;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _ch_grp_0_src(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0xFL << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _ch_grp_src(dev, arg, 0, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _ch_grp_1_src(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0xFL << SHIFT)
	#define	SHIFT	4

	int	ret;

	ret	= _ch_grp_src(dev, arg, 1, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _ch_grp_2_src(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0xFL << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _ch_grp_src(dev, arg, 2, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _ch_grp_3_src(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0xFL << SHIFT)
	#define	SHIFT	12

	int	ret;

	ret	= _ch_grp_src(dev, arg, 3, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _channel_order(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	16

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_CHANNEL_ORDER_ASYNC:
		case SDI_CHANNEL_ORDER_SYNC:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	4

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_DATA_FORMAT_2S_COMP:
		case SDI_DATA_FORMAT_OFF_BIN:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _init_mode(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	5

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_INIT_MODE_TARGET:
		case SDI_INIT_MODE_INITIATOR:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			SDI_IO_TIMEOUT_MIN,
			SDI_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _sw_sync(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	6

	int	ret;
	u32	v;

	v	= 0x1 << SHIFT;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
	ret	= _wait_for_ready(dev, 1000);

	return(ret);
}



//*****************************************************************************
static int _sw_sync_mode(dev_data_t* dev, s32* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1L << SHIFT)
	#define	SHIFT	17

	int	ret;
	u32	v;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			ret		= 0;
			arg[0]	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
			arg[0]	&= MASK;
			arg[0]	>>= SHIFT;
			break;

		case SDI_SW_SYNC_MODE_SW_SYNC:
		case SDI_SW_SYNC_MODE_CLR_BUF:

			v	= arg[0] << SHIFT;
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, v, MASK);
			ret	= _wait_for_ready(dev, 1000);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _rate_div_ndiv(
	dev_data_t*	dev,
	s32*		arg,
	int			channel,
	VADDR_T		vaddr,
	u32			mask,
	u32			shift)
{
	#undef	MASK
	#undef	SHIFT

	int	ret;
	u32	v;

	if ((channel < 0) || (channel >= dev->cache.channel_qty))
	{
		ret	= -EINVAL;
	}
	else if (arg[0] == -1)
	{
		ret		= 0;
		arg[0]	= os_reg_mem_rx_u32(dev, vaddr);
		arg[0]	&= mask;
		arg[0]	>>= shift;
	}
	else if (	(arg[0] < dev->cache.rate_div_ndiv_min)	||
				(arg[0] > dev->cache.rate_div_ndiv_max))
	{
		ret	= -EINVAL;
	}
	else
	{
		v	= arg[0] << shift;
		os_reg_mem_mx_u32(dev, vaddr, v, mask);
		ret	= _wait_for_ready(dev, 1000);
	}

	return(ret);
}



//*****************************************************************************
static int _rate_div_00_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 0, dev->vaddr.gsc_rd01r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_01_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 1, dev->vaddr.gsc_rd01r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_02_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 2, dev->vaddr.gsc_rd23r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_03_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 3, dev->vaddr.gsc_rd23r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_04_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 4, dev->vaddr.gsc_rd45r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_05_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 5, dev->vaddr.gsc_rd45r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_06_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 6, dev->vaddr.gsc_rd67r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_07_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 7, dev->vaddr.gsc_rd67r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_08_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 8, dev->vaddr.gsc_rd89r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_09_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 9, dev->vaddr.gsc_rd89r_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_10_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 10, dev->vaddr.gsc_rdabr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_11_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 11, dev->vaddr.gsc_rdabr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_12_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 12, dev->vaddr.gsc_rdcdr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_13_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 13, dev->vaddr.gsc_rdcdr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_14_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 14, dev->vaddr.gsc_rdefr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_div_15_ndiv(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x3F << SHIFT)
	#define	SHIFT	8

	int	ret;

	ret	= _rate_div_ndiv(dev, arg, 15, dev->vaddr.gsc_rdefr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_gen_nrate(
	dev_data_t*	dev,
	s32*		arg,
	int			rate_gen,
	VADDR_T		vaddr,
	u32			mask,
	u32			shift)
{
	#undef	MASK
	#undef	SHIFT

	int	ret;
	u32	v;

	if (arg[0] == -1)
	{
		ret		= 0;
		arg[0]	= os_reg_mem_rx_u32(dev, vaddr);
		arg[0]	&= mask;
		arg[0]	>>= shift;
	}
	else if (	(rate_gen < 0)								||
				(rate_gen >= dev->cache.rate_gen_qty)		||
				(arg[0] < dev->cache.rate_gen_nrate_min)	||
				(arg[0] > dev->cache.rate_gen_nrate_max))
	{
		ret	= -EINVAL;
	}
	else
	{
		v	= arg[0] << shift;
		os_reg_mem_mx_u32(dev, vaddr, v, mask);
		ret	= _wait_for_ready(dev, 1000);
	}

	return(ret);

}



//*****************************************************************************
static int _rate_gen_a_nrate(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1FF << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_gen_nrate(dev, arg, 0, dev->vaddr.gsc_rcar_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_gen_b_nrate(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1FF << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_gen_nrate(dev, arg, 1, dev->vaddr.gsc_rcbr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_gen_c_nrate(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1FF << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_gen_nrate(dev, arg, 2, dev->vaddr.gsc_rccr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rate_gen_d_nrate(dev_data_t* dev, void* arg)
{
	#undef	MASK
	#undef	SHIFT
	#define	MASK	(0x1FF << SHIFT)
	#define	SHIFT	0

	int	ret;

	ret	= _rate_gen_nrate(dev, arg, 3, dev->vaddr.gsc_rcdr_32, MASK, SHIFT);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _irq_sel(dev_data_t* dev, s32* arg)
{
	u32	reg;
	int	ret	= 0;

	if (arg[0] == -1)
	{
		// Retrieve the current setting.
		reg		= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);
		arg[0]	= GSC_FIELD_DECODE(reg, 10, 8);
	}
	else
	{
		// Validate the option value passed in.

		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case SDI_IRQ_INIT_DONE:
			case SDI_IRQ_AUTO_CAL_DONE:
			case SDI_IRQ_CHANNELS_READY:
			case SDI_IRQ_BUF_THRESH_L2H:
			case SDI_IRQ_BUF_THRESH_H2L:
			case SDI_IRQ_BUF_AE:
			case SDI_IRQ_BUF_AF:

				break;
		}

		if (ret == 0)
		{
			// Clear the status bit and apply the new setting.
			os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bcr_32, arg[0] << 8, 0xF00);
		}
	}

	return(ret);
}



//*****************************************************************************
static int _channels_ready(dev_data_t* dev, s32* arg)
{
	u32	bcr;
	int	ret		= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case SDI_CHANNELS_READY_WAIT:

			ret	= _wait_for_ready(dev, 1000);

			if (ret)
				break;

			FALLTHROUGH;	/* fall through */

		case -1:

			bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

			if (bcr & D13)
				arg[0]	= SDI_CHANNELS_READY_YES;
			else
				arg[0]	= SDI_CHANNELS_READY_NO;

			break;
	}

	return(ret);
}



//*****************************************************************************
static int _ain_buf_thresh_sts(dev_data_t* dev, s32* arg)
{
	u32	bcr;

	bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

	if (bcr & D14)
		arg[0]	= SDI_AIN_BUF_THRESH_STS_SET;
	else
		arg[0]	= SDI_AIN_BUF_THRESH_STS_CLEAR;

	return(0);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ SDI_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ SDI_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ SDI_IOCTL_REG_MOD,			(void*) gsc_reg_mod_ioctl		},
	{ SDI_IOCTL_QUERY,				(void*) _query					},
	{ SDI_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ SDI_IOCTL_AUTO_CALIBRATE,		(void*) _auto_calibrate			},
	{ SDI_IOCTL_RX_IO_MODE,			(void*) _rx_io_mode				},
	{ SDI_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout			},
	{ SDI_IOCTL_AIN_BUF_CLEAR,		(void*) _ain_buf_clear			},
	{ SDI_IOCTL_AIN_BUF_INPUT,		(void*) _ain_buf_input			},
	{ SDI_IOCTL_AIN_BUF_THRESH,		(void*) _ain_buf_thresh			},
	{ SDI_IOCTL_AIN_MODE,			(void*) _ain_mode				},
	{ SDI_IOCTL_AIN_RANGE,			(void*) _ain_range				},
	{ SDI_IOCTL_CH_GRP_0_SRC,		(void*) _ch_grp_0_src			},
	{ SDI_IOCTL_CH_GRP_1_SRC,		(void*) _ch_grp_1_src			},
	{ SDI_IOCTL_CH_GRP_2_SRC,		(void*) _ch_grp_2_src			},
	{ SDI_IOCTL_CH_GRP_3_SRC,		(void*) _ch_grp_3_src			},
	{ SDI_IOCTL_DATA_FORMAT,		(void*) _data_format			},
	{ SDI_IOCTL_INIT_MODE,			(void*) _init_mode				},
	{ SDI_IOCTL_CHANNEL_ORDER,		(void*) _channel_order			},
	{ SDI_IOCTL_SW_SYNC,			(void*) _sw_sync				},
	{ SDI_IOCTL_SW_SYNC_MODE,		(void*) _sw_sync_mode			},
	{ SDI_IOCTL_RATE_DIV_00_NDIV,	(void*) _rate_div_00_ndiv		},
	{ SDI_IOCTL_RATE_DIV_01_NDIV,	(void*) _rate_div_01_ndiv		},
	{ SDI_IOCTL_RATE_DIV_02_NDIV,	(void*) _rate_div_02_ndiv		},
	{ SDI_IOCTL_RATE_DIV_03_NDIV,	(void*) _rate_div_03_ndiv		},
	{ SDI_IOCTL_RATE_DIV_04_NDIV,	(void*) _rate_div_04_ndiv		},
	{ SDI_IOCTL_RATE_DIV_05_NDIV,	(void*) _rate_div_05_ndiv		},
	{ SDI_IOCTL_RATE_DIV_06_NDIV,	(void*) _rate_div_06_ndiv		},
	{ SDI_IOCTL_RATE_DIV_07_NDIV,	(void*) _rate_div_07_ndiv		},
	{ SDI_IOCTL_RATE_DIV_08_NDIV,	(void*) _rate_div_08_ndiv		},
	{ SDI_IOCTL_RATE_DIV_09_NDIV,	(void*) _rate_div_09_ndiv		},
	{ SDI_IOCTL_RATE_DIV_10_NDIV,	(void*) _rate_div_10_ndiv		},
	{ SDI_IOCTL_RATE_DIV_11_NDIV,	(void*) _rate_div_11_ndiv		},
	{ SDI_IOCTL_RATE_DIV_12_NDIV,	(void*) _rate_div_12_ndiv		},
	{ SDI_IOCTL_RATE_DIV_13_NDIV,	(void*) _rate_div_13_ndiv		},
	{ SDI_IOCTL_RATE_DIV_14_NDIV,	(void*) _rate_div_14_ndiv		},
	{ SDI_IOCTL_RATE_DIV_15_NDIV,	(void*) _rate_div_15_ndiv		},
	{ SDI_IOCTL_RATE_GEN_A_NRATE,	(void*) _rate_gen_a_nrate		},
	{ SDI_IOCTL_RATE_GEN_B_NRATE,	(void*) _rate_gen_b_nrate		},
	{ SDI_IOCTL_RATE_GEN_C_NRATE,	(void*) _rate_gen_c_nrate		},
	{ SDI_IOCTL_RATE_GEN_D_NRATE,	(void*) _rate_gen_d_nrate		},
	{ SDI_IOCTL_RX_IO_ABORT,		(void*) _rx_io_abort			},
	{ SDI_IOCTL_IRQ_SEL,			(void*) _irq_sel				},
	{ SDI_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ SDI_IOCTL_WAIT_CANCEL,		(void*) gsc_wait_cancel_ioctl	},
	{ SDI_IOCTL_WAIT_STATUS,		(void*) gsc_wait_status_ioctl	},
	{ SDI_IOCTL_AUTO_CAL_STATUS,	(void*) _auto_cal_status		},
	{ SDI_IOCTL_CHANNELS_READY,		(void*) _channels_ready			},
	{ SDI_IOCTL_AIN_BUF_THRESH_STS,	(void*) _ain_buf_thresh_sts		},
	{ -1, NULL }
};



//*****************************************************************************
static int _wait_for_ready(dev_data_t* dev, long ms_limit)
{
	u32				bcr;
	int				i;
	unsigned long	limit;
	int				ret;

	limit	= os_time_tick_get() + os_time_ms_to_ticks(ms_limit);

	for (;;)
	{
		// Read the register so we can check.
		bcr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bcr_32);

		if (bcr & CHAN_READY)
		{
			ret	= 0;
			break;
		}

		if (os_time_tick_timedout(limit))
		{
			// We've waited long enough.
			ret	= -EIO;
			break;
		}

		i	= os_time_tick_sleep(1);

		if (i)
		{
			// The timeout returned prematurely; it was signaled.
			// We've essentially been told to quit.
			ret	= -ECANCELED;
			break;
		}
	}

	return(ret);
}


