// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/ain_buf.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ain_buf_clear_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_AIN_BUF_CLEAR.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_clear_test(int fd)
{
	#define	THRESHOLD_FLAG	(1 << 14)

	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{				// Wait for the buffer to fill.
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	6,				// 256K / (25K * 2)
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Verify that we got lots of input.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	SDI_GSC_BCR,
			/* mask		*/	THRESHOLD_FLAG,
			/* value	*/	THRESHOLD_FLAG
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	SDI_GSC_BSR,
			/* mask		*/	0xFFFF,
			/* value	*/	0x0000
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	gsc_label("SDI_IOCTL_AIN_BUF_CLEAR");
	errs	= service_ioctl_set_reg_list(fd, list);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	ain_buf_input_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_AIN_BUF_INPUT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_input_test(int fd)
{
	#define	CONTROL_BIT		(1 << 18)

	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	CONTROL_BIT,
			/* value	*/	CONTROL_BIT
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	CONTROL_BIT,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	CONTROL_BIT,
			/* value	*/	CONTROL_BIT
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	CONTROL_BIT,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	gsc_label("SDI_IOCTL_AIN_BUF_INPUT");
	errs	= service_ioctl_set_reg_list(fd, list);

	if (errs == 0)
		errs	= service_ioctl_reg_get_list(fd, list);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_ain_buf_threshold_tests
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_AIN_BUF_THRESH.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		size	The size of the device's FIFO.
*
*		mask	The threshold mask.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _ain_buf_threshold_tests(int fd, s32 size, s32 mask)
{
	service_data_t	list_1[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x0000,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x0000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x000F,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x000F
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x00F0,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x00F0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x0F00,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x0F00
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0xF000,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0xF000
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	service_data_t	list_2[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x30000,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x30000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	0x3F000,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	0x3F000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	mask,
			/* reg		*/	SDI_GSC_BTR,
			/* mask		*/	mask,
			/* value	*/	mask
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list_1);

	if (errs == 0)
		errs	= service_ioctl_reg_get_list(fd, list_1);

	if ((errs == 0) && (size >= 0x3FFFF))
	{
		errs	= service_ioctl_set_reg_list(fd, list_2);

		if (errs == 0)
			errs	= service_ioctl_reg_get_list(fd, list_2);
	}

	return(errs);
}


/******************************************************************************
*
*	Function:	ain_buf_threshold_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_AIN_BUF_THRESH.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ain_buf_threshold_test(int fd)
{
	int	errs;
	s32	mask	= 0;
	s32	size	= 0;

	gsc_label("SDI_IOCTL_AIN_BUF_THRESH");
	errs	= sdi_query(fd, -1, 0, SDI_QUERY_FIFO_SIZE, &size);

	if (errs == 0)
		errs	= sdi_query(fd, -1, 0, SDI_QUERY_THRESHOLD_MASK, &mask);

	if (errs == 0)
		errs	= _ain_buf_threshold_tests(fd, size, mask);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


