// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/rate_div.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_rate_div_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_XX_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		reg		The register where this Ndiv is stored.
*
*		shift	The offset of the Ndiv field in the above register.
*
*		ndiv	The Ndiv value to test.
*
*		mask	The mask of valid Ndiv bits.
*
*		min		The minimum Ndiv value for this device.
*
*		max		The maximum Ndiv value for this device.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _rate_div_ndiv_test(
	int		fd,
	long	cmd,
	u32		reg,
	int		shift,
	s32		ndiv,
	s32		mask,
	s32		min,
	s32		max)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	min,
			/* reg		*/	reg,
			/* mask		*/	mask << shift,
			/* value	*/	min << shift
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	ndiv,
			/* reg		*/	reg,
			/* mask		*/	mask << shift,
			/* value	*/	ndiv << shift
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	max,
			/* reg		*/	reg,
			/* mask		*/	mask << shift,
			/* value	*/	max << shift
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);

	if (errs == 0)
		errs	= service_ioctl_reg_get_list(fd, list);

	return(errs);
}



/******************************************************************************
*
*	Function:	_rate_div_ndiv_tests
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_XX_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		name	A text version of the above command code.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _rate_div_ndiv_tests(int fd, long cmd, const char* name)
{
	static const struct
	{
		u32	reg;
		int	shift;
	} list[16]	=
	{	// reg				shift
		{ SDI_GSC_RD01R,	0	},
		{ SDI_GSC_RD01R,	8	},
		{ SDI_GSC_RD23R,	0	},
		{ SDI_GSC_RD23R,	8	},
		{ SDI_GSC_RD45R,	0	},
		{ SDI_GSC_RD45R,	8	},
		{ SDI_GSC_RD67R,	0	},
		{ SDI_GSC_RD67R,	8	},
		{ SDI_GSC_RD89R,	0	},
		{ SDI_GSC_RD89R,	8	},
		{ SDI_GSC_RDABR,	0	},
		{ SDI_GSC_RDABR,	8	},
		{ SDI_GSC_RDCDR,	0	},
		{ SDI_GSC_RDCDR,	8	},
		{ SDI_GSC_RDEFR,	0	},
		{ SDI_GSC_RDEFR,	8	}
	};

	int	errs	= 0;
	int	chan	= 0;
	s32	chans	= 0;
	s32	mask	= 0;
	s32	max		= 0;
	s32	min		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label(name);

		switch (cmd)
		{
			default:

				errs	= 1;
				printf(	"FAIL <---  (%d. INTERNAL ERROR, cmd 0x%lX)\n",
						__LINE__,
						(long) cmd);
				break;

			case SDI_IOCTL_RATE_DIV_00_NDIV:	chan	= 0;	break;
			case SDI_IOCTL_RATE_DIV_01_NDIV:	chan	= 1;	break;
			case SDI_IOCTL_RATE_DIV_02_NDIV:	chan	= 2;	break;
			case SDI_IOCTL_RATE_DIV_03_NDIV:	chan	= 3;	break;
			case SDI_IOCTL_RATE_DIV_04_NDIV:	chan	= 4;	break;
			case SDI_IOCTL_RATE_DIV_05_NDIV:	chan	= 5;	break;
			case SDI_IOCTL_RATE_DIV_06_NDIV:	chan	= 6;	break;
			case SDI_IOCTL_RATE_DIV_07_NDIV:	chan	= 7;	break;
			case SDI_IOCTL_RATE_DIV_08_NDIV:	chan	= 8;	break;
			case SDI_IOCTL_RATE_DIV_09_NDIV:	chan	= 9;	break;
			case SDI_IOCTL_RATE_DIV_10_NDIV:	chan	= 10;	break;
			case SDI_IOCTL_RATE_DIV_11_NDIV:	chan	= 11;	break;
			case SDI_IOCTL_RATE_DIV_12_NDIV:	chan	= 12;	break;
			case SDI_IOCTL_RATE_DIV_13_NDIV:	chan	= 13;	break;
			case SDI_IOCTL_RATE_DIV_14_NDIV:	chan	= 14;	break;
			case SDI_IOCTL_RATE_DIV_15_NDIV:	chan	= 15;	break;
		}

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &chans);

		if ((errs == 0) && (chan >= chans))
		{
			printf("SKIPPED  (This isn't supported on this device.)\n");
			break;
		}

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_NDIV_MASK, &mask);

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_NDIV_MAX, &max);

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_NDIV_MIN, &min);

		if (errs == 0)
		{
			errs	= _rate_div_ndiv_test(
						fd,
						cmd,
						list[chan].reg,
						list[chan].shift,
						min + 1,
						mask,
						min,
						max);
		}

		if (errs == 0)
		{
			errs	= _rate_div_ndiv_test(
						fd,
						cmd,
						list[chan].reg,
						list[chan].shift,
						(min + max) / 2,
						mask,
						min,
						max);
		}

		if (errs == 0)
		{
			errs	= _rate_div_ndiv_test(
						fd,
						cmd,
						list[chan].reg,
						list[chan].shift,
						max - 1,
						mask,
						min,
						max);
		}

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_00_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_00_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_00_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_00_NDIV,
				"SDI_IOCTL_RATE_DIV_00_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_01_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_01_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_01_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_01_NDIV,
				"SDI_IOCTL_RATE_DIV_01_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_02_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_02_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_02_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_02_NDIV,
				"SDI_IOCTL_RATE_DIV_02_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_03_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_03_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_03_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_03_NDIV,
				"SDI_IOCTL_RATE_DIV_03_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_04_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_04_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_04_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_04_NDIV,
				"SDI_IOCTL_RATE_DIV_04_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_05_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_05_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_05_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_05_NDIV,
				"SDI_IOCTL_RATE_DIV_05_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_06_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_06_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_06_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_06_NDIV,
				"SDI_IOCTL_RATE_DIV_06_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_07_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_07_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_07_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_07_NDIV,
				"SDI_IOCTL_RATE_DIV_07_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_08_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_08_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_08_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_08_NDIV,
				"SDI_IOCTL_RATE_DIV_08_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_09_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_09_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_09_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_09_NDIV,
				"SDI_IOCTL_RATE_DIV_09_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_10_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_10_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_10_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_10_NDIV,
				"SDI_IOCTL_RATE_DIV_10_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_11_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_11_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_11_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_11_NDIV,
				"SDI_IOCTL_RATE_DIV_11_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_12_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_12_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_12_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_12_NDIV,
				"SDI_IOCTL_RATE_DIV_12_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_13_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_13_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_13_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_13_NDIV,
				"SDI_IOCTL_RATE_DIV_13_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_14_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_14_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_14_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_14_NDIV,
				"SDI_IOCTL_RATE_DIV_14_NDIV");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_div_15_ndiv_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_DIV_15_NDIV.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_div_15_ndiv_test(int fd)
{
	int	errs;

	errs	= _rate_div_ndiv_tests(
				fd,
				SDI_IOCTL_RATE_DIV_15_NDIV,
				"SDI_IOCTL_RATE_DIV_15_NDIV");
	return(errs);
}


