// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/rate_gen.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_rate_gen_nrate_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_X_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		reg		The register where this Nrate is stored.
*
*		nrate	The Nrate value to test.
*
*		min		The minimum Nrate value for this device.
*
*		max		The maximum Nrate value for this device.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _rate_gen_nrate_test(
	int		fd,
	long	cmd,
	u32		reg,
	s32		nrate,
	s32		min,
	s32		max)
{
	#define	MASK	0x1FF

	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	min,
			/* reg		*/	reg,
			/* mask		*/	MASK,
			/* value	*/	min
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	nrate,
			/* reg		*/	reg,
			/* mask		*/	MASK,
			/* value	*/	nrate
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	cmd,
			/* arg		*/	max,
			/* reg		*/	reg,
			/* mask		*/	MASK,
			/* value	*/	max
		},

		{
			/* service	*/	SERVICE_IOCTL_SET,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;

	errs	= service_ioctl_set_reg_list(fd, list);

	if (errs == 0)
		errs	= service_ioctl_reg_get_list(fd, list);

	return(errs);
}



/******************************************************************************
*
*	Function:	_rate_gen_nrate_tests
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_X_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		cmd		The IOCTL command code to test.
*
*		name	A text version of the above command code.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _rate_gen_nrate_tests(int fd, long cmd, const char* name)
{
	static const struct
	{
		u32	reg;
	} list[4]	=
	{	// reg
		{ SDI_GSC_RCAR	},
		{ SDI_GSC_RCBR	},
		{ SDI_GSC_RCCR	},
		{ SDI_GSC_RCDR	}
	};

	int	errs	= 0;
	int	gen		= 0;
	s32	gens	= 0;
	s32	max		= 0;
	s32	min		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label(name);

		switch (cmd)
		{
			default:

				errs	= 1;
				printf(	"FAIL <---  (%d. INTERNAL ERROR, cmd 0x%lX)\n",
						__LINE__,
						(long) cmd);
				break;

			case SDI_IOCTL_RATE_GEN_A_NRATE:	gen	= 0;	break;
			case SDI_IOCTL_RATE_GEN_B_NRATE:	gen	= 1;	break;
			case SDI_IOCTL_RATE_GEN_C_NRATE:	gen	= 2;	break;
			case SDI_IOCTL_RATE_GEN_D_NRATE:	gen	= 3;	break;
		}

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_RATE_GEN_QTY, &gens);

		if ((errs == 0) && (gen >= gens))
		{
			printf("SKIPPED  (This isn't supported on this device.)\n");
			break;
		}

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MAX, &max);

		if (errs == 0)
			errs	= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MIN, &min);

		if (errs == 0)
		{
			errs	= _rate_gen_nrate_test(
						fd,
						cmd,
						list[gen].reg,
						min + 1,
						min,
						max);
		}

		if (errs == 0)
		{
			errs	= _rate_gen_nrate_test(
						fd,
						cmd,
						list[gen].reg,
						(min + max) / 2,
						min,
						max);
		}

		if (errs == 0)
		{
			errs	= _rate_gen_nrate_test(
						fd,
						cmd,
						list[gen].reg,
						max - 1,
						min,
						max);
		}

		if (errs == 0)
			printf("PASS\n");

		break;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	rate_gen_a_nrate_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_A_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_gen_a_nrate_test(int fd)
{
	int	errs;

	errs	= _rate_gen_nrate_tests(
				fd,
				SDI_IOCTL_RATE_GEN_A_NRATE,
				"SDI_IOCTL_RATE_GEN_A_NRATE");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_gen_b_nrate_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_B_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_gen_b_nrate_test(int fd)
{
	int	errs;

	errs	= _rate_gen_nrate_tests(
				fd,
				SDI_IOCTL_RATE_GEN_B_NRATE,
				"SDI_IOCTL_RATE_GEN_B_NRATE");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_gen_c_nrate_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_C_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_gen_c_nrate_test(int fd)
{
	int	errs;

	errs	= _rate_gen_nrate_tests(
				fd,
				SDI_IOCTL_RATE_GEN_C_NRATE,
				"SDI_IOCTL_RATE_GEN_C_NRATE");
	return(errs);
}



/******************************************************************************
*
*	Function:	rate_gen_d_nrate_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_RATE_GEN_D_NRATE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int rate_gen_d_nrate_test(int fd)
{
	int	errs;

	errs	= _rate_gen_nrate_tests(
				fd,
				SDI_IOCTL_RATE_GEN_D_NRATE,
				"SDI_IOCTL_RATE_GEN_D_NRATE");
	return(errs);
}


