// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/sw_sync.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test_sw_sync(int fd)
{
	// There are no settings to validate.
	sdi_initialize(fd, -1, 0);
	return(0);
}



//*****************************************************************************
static int _function_test_sw_sync(int fd)
{
	// This is tested via the SYNC MODE test.
	sdi_initialize(fd, -1, 0);
	return(0);
}



/******************************************************************************
*
*	Function:	sw_sync_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_SW_SYNC.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sw_sync_test(int fd)
{
	int	errs	= 0;

	gsc_label("SDI_IOCTL_SW_SYNC");
	errs	+= _service_test_sw_sync(fd);
	errs	+= _function_test_sw_sync(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _service_test_sw_sync_mode(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC_MODE,
			/* arg		*/	SDI_SW_SYNC_MODE_SW_SYNC,
			/* reg		*/	SDI_GSC_BCR,
			/* mask		*/	0x20000,
			/* value	*/	0x00000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC_MODE,
			/* arg		*/	SDI_SW_SYNC_MODE_CLR_BUF,
			/* reg		*/	SDI_GSC_BCR,
			/* mask		*/	0x20000,
			/* value	*/	0x20000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= sdi_initialize(fd, -1, 0);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= service_ioctl_reg_get_list(fd, list);

	errs	+= sdi_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test_sw_sync_mode(int fd)
{
	static const service_data_t	list[]	=
	{
		{	// 0
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 1
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_CH_GRP_0_SRC,
			/* arg		*/	SDI_CH_GRP_SRC_GEN_A,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 2
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_INIT_MODE,
			/* arg		*/	SDI_INIT_MODE_INITIATOR,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 3
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_DIV_00_NDIV,
			/* arg		*/	2,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 4
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_GEN_A_NRATE,
			/* arg		*/	307,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 5
			/* service	*/	SERVICE_SLEEP,
			/* cmd		*/	0,
			/* arg		*/	2,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 6
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 7
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	SDI_GSC_BSR,
			/* mask		*/	0xFFFF,
			/* value	*/	0xFFFF
		},
		// Verify that the SW SYNC mode doesn't clear the buffer.
		{	// 8
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC_MODE,
			/* arg		*/	SDI_SW_SYNC_MODE_SW_SYNC,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 9
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 10
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	SDI_GSC_BSR,
			/* mask		*/	0xFFFF,
			/* value	*/	0xFFFF
		},
		// Verify that the CLEAR BUFFER mode does clear the buffer.
		{	// 11
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC_MODE,
			/* arg		*/	SDI_SW_SYNC_MODE_CLR_BUF,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 12
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_SW_SYNC,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{	// 13
			// The buffer should have been cleared, but sholdn't have enough
			// time to fill up again.
			/* service	*/	SERVICE_REG_TEST,
			/* cmd		*/	0,
			/* arg		*/	0,
			/* reg		*/	SDI_GSC_BSR,
			/* mask		*/	0x8000,
			/* value	*/	0x0000
		},

		{ SERVICE_END_LIST }
	};

	int	errs	= 0;

	errs	+= sdi_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_reg_list(fd, list);
	errs	+= sdi_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	sw_sync_mode_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_SW_SYNC_MODE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sw_sync_mode_test(int fd)
{
	int	errs	= 0;

	gsc_label("SDI_IOCTL_SW_SYNC_MODE");
	errs	+= _service_test_sw_sync_mode(fd);
	errs	+= _function_test_sw_sync_mode(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


