// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/samples/sbtest/wait_event.c $
// $Rev: 51952 $
// $Date: 2022-10-25 08:36:47 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC		(1000L)
#define	_80_SEC		(80L * 1000L)
#define	_90_SEC		(90L * 1000L)



// variables ******************************************************************

static	int			rx_stat;
static	sem_t		sem_enter;
static	sem_t		sem_exit;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(int fd)
{
	// There are no settings bits to test here.
	sdi_initialize(fd, -1, 0);
	return(0);
}



//*****************************************************************************
static int	_wait_main_dma(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select DMA mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_BMDMA,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Fill the FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_GEN_A_NRATE,
			/* arg		*/	511,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_DIV_00_NDIV,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= sdi_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, list);

		// Now perform a DMA read to trigger the wait.

		if (errs == 0)
		{
			ret		= sdi_read(fd, buf, sizeof(buf));
			errs	+= (ret == sizeof(buf)) ? 0 : 1;
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_main_gsc(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_init_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_auto_cal_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AUTO_CALIBRATE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_channels_ready(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_IRQ_SEL,
			/* arg		*/	SDI_IRQ_CHANNELS_READY,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_buf_thresh_l2h(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_IRQ_SEL,
			/* arg		*/	SDI_IRQ_BUF_THRESH_L2H,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_buf_thresh_h2l(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_IRQ_SEL,
			/* arg		*/	SDI_IRQ_BUF_THRESH_H2L,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_THRESH,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int	errs;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	= service_ioctl_set_reg_list(fd, list);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_buf_ae(int fd, int supported)
{
	// Not implemented.
	return(0);
}



//*****************************************************************************
static int	_wait_gsc_buf_af(int fd, int supported)
{
	// Not implemented.
	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_done(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		// Fill the FIFO.

		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_ENABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_GEN_A_NRATE,
			/* arg		*/	511,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RATE_DIV_00_NDIV,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{				// Wait to see if we get any data.
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= sdi_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, list);

		// Now perform a DMA read to trigger the wait.

		if (errs == 0)
		{
			ret		= sdi_read(fd, buf, sizeof(buf));
			errs	+= (ret == sizeof(buf)) ? 0 : 1;
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_error(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		// Now perform a read to trigger the wait.
		ret	= sdi_read(fd, NULL, 3);

		if (ret >= 0)
		{
			errs++;
			printf("FAIL <---  (%d. expected < 0, got %d)\n", __LINE__, ret);
		}

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_timeout(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= sdi_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, list);

		// Now perform a read to trigger the wait.

		if (errs == 0)
		{
			ret	= sdi_read(fd, buf, sizeof(buf));

			if (ret != 0)
			{
				errs++;
				printf(	"FAIL <---  (%d. expected 0, got %d)\n",
						__LINE__,
						ret);
			}
		}

		errs	+= sdi_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	u32		buf[1024];
	int		fd	= (int) (long) arg;
	long	ret;

	ret	= sdi_read(fd, buf, sizeof(buf));

	if (ret == 0)
	{
		rx_stat	= 1;
	}
	else
	{
		rx_stat	= 2;
		printf("FAIL <---  (%d. sdi_read returned %ld, expected 0)\n",
				__LINE__,
				(long) ret);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd, int supported)
{
	static const service_data_t	list[]	=
	{
		{					// Select PIO mode.
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_MODE,
			/* arg		*/	GSC_IO_MODE_PIO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_RX_IO_TIMEOUT,
			/* arg		*/	10,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_INPUT,
			/* arg		*/	SDI_AIN_BUF_INPUT_DISABLE,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	SDI_IOCTL_AIN_BUF_CLEAR,
			/* arg		*/	1,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},

		{ SERVICE_END_LIST }
	};

	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= sdi_initialize(fd, -1, 0);
		errs	+= service_ioctl_set_reg_list(fd, list);

		// Now start the read thread so we can do the abort.
		memset(&thread, 0, sizeof(thread));

		if (errs == 0)
		{
			errs	+= os_thread_create(	&thread,
											"Rx",
											_read_thread,
											(void*) (long) fd);
		}

		for (i = 0; (errs == 0) && (i < 100); i++)
		{
			set		= 0;
			ret		= sdi_ioctl(fd, SDI_IOCTL_RX_IO_ABORT, &set);
			errs	+= ret ? 1 : 0;

			if (errs)
				break;

			if (set)
				break;

			os_sleep_ms(100);
		}

		// Wait for the thread to report its status.

		for (i = 0; i < 100; i++)
		{
			if (errs)
				break;

			if (rx_stat == 0)
			{
				os_sleep_ms(100);
				continue;
			}

			if (rx_stat == 1)
				break;

			errs++;
			break;
		}

		errs	+= os_thread_destroy(&thread);
		errs	+= sdi_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_timeout(int fd, int supported)
{
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		os_sleep_ms(2000);
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_cancel(int fd, int supported)
{
	int			errs;
	int			ret;
	gsc_wait_t	wait;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		wait.flags			= 0;
		wait.main			= 0xFFFFFFFF;
		wait.gsc			= 0xFFFFFFFF;
		wait.alt			= 0xFFFFFFFF;
		wait.io				= 0xFFFFFFFF;
		wait.timeout_ms		= 0;
		wait.count			= 0;
		ret		= sdi_ioctl(wait_fd, SDI_IOCTL_WAIT_CANCEL, &wait);
		errs	= ret ? 1 : 0;
		ret		= errs;
	}

	return(ret);
}



// variables ******************************************************************

static const struct
{
	const char*	name;
	int			(*function)(int fd, int supported);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name						function					flags			main							...			timeout_ms
	{ "DMA0/DMA1",				_wait_main_dma,				{ 0,			GSC_WAIT_MAIN_DMA0 |
																			GSC_WAIT_MAIN_DMA1,				0, 0, 0,	_80_SEC	}	},
	{ "GSC",					_wait_main_gsc,				{ 0,			GSC_WAIT_MAIN_GSC,				0, 0, 0,	_80_SEC	}	},

	// name						function					flags ...		gsc								...			timeout_ms
	{ "Initialization Done",	_wait_gsc_init_done,		{ 0, 0,			SDI_WAIT_GSC_INIT_DONE,			0, 0,		_80_SEC	}	},
	{ "Auto-Cal Done",			_wait_gsc_auto_cal_done,	{ 0, 0,			SDI_WAIT_GSC_AUTO_CAL_DONE,		0, 0,		_80_SEC	}	},
	{ "Channels Ready",			_wait_gsc_channels_ready,	{ 0, 0,			SDI_WAIT_GSC_CHANNELS_READY,	0, 0,		_80_SEC	}	},
	{ "Buf Threshold Low->Hi",	_wait_gsc_buf_thresh_l2h,	{ 0, 0,			SDI_WAIT_GSC_BUF_THRESH_L2H,	0, 0,		_80_SEC	}	},
	{ "Buf Threshold Hi->Low",	_wait_gsc_buf_thresh_h2l,	{ 0, 0,			SDI_WAIT_GSC_BUF_THRESH_H2L,	0, 0,		_80_SEC	}	},
	{ "Buffer Almost Empty",	_wait_gsc_buf_ae,			{ 0, 0,			SDI_WAIT_GSC_BUF_AE,			0, 0,		_80_SEC	}	},
	{ "Buffer Almost Full",		_wait_gsc_buf_af,			{ 0, 0,			SDI_WAIT_GSC_BUF_AF,			0, 0,		_80_SEC	}	},

	// name						function					flags ...		io											timeout_ms
	{ "Rx Done",				_wait_io_rx_done,			{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_DONE,						_80_SEC	}	},
	{ "Rx Error",				_wait_io_rx_error,			{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_ERROR,						_80_SEC	}	},
	{ "Rx Timeout",				_wait_io_rx_timeout,		{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_TIMEOUT,						_80_SEC	}	},
	{ "Rx Abort",				_wait_io_rx_abort,			{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_ABORT,						_80_SEC	}	},

	// name						function					flags ...		io											timeout_ms
	{ "Wait Timeout",			_wait_timeout,				{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_DONE,						_1_SEC	}	},
	{ "Wait Cancel",			_wait_cancel,				{ 0, 0, 0, 0,	SDI_WAIT_IO_RX_DONE,						_90_SEC	}	},

	// terminate list
	{ NULL, NULL, { 1 } }
};



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	ret;

	sem_post(&sem_enter);
	ret	= sdi_ioctl(wait_fd, SDI_IOCTL_WAIT_EVENT, &wait_dst);

	if (ret)
		wait_dst.flags	= 0xFFFFFFFF;

	sem_post(&sem_exit);
	return(0);
}



//*****************************************************************************
static int _process_thread(int index)
{
	char		buf[64];
	int			errs;
	int			i;
	int			ret;
	os_thread_t	thread;
	gsc_wait_t	wait;

	for (;;)	// A convenience loop.
	{
		sem_init(&sem_enter, 0, 0);
		sem_init(&sem_exit, 0, 0);
		memset(&thread, 0, sizeof(thread));
		sprintf(buf, "wait event %d", index);
		wait_dst	= wait_src[index].wait;
		errs		= os_thread_create(&thread, buf, _wait_thread, NULL);

		// Wait for the thread to become waiting.
		sem_wait(&sem_enter);

		if (errs == 0)
		{
			wait	= wait_src[index].wait;

			for (i = 0; i < 100; i++)
			{
				ret	= sdi_ioctl(wait_fd, SDI_IOCTL_WAIT_STATUS, &wait);

				if (ret)
				{
					printf(	"FAIL <---  (%d. status request %d: error %d)\n",
							__LINE__,
							index,
							ret);
					errs++;
					break;
				}

				if (wait.count == 0)
				{
					os_sleep_ms(100);
					continue;
				}

				if (wait.count == 1)
					break;

				errs++;
				printf(	"FAIL <---  (%d. invalid wait count %ld)\n",
						__LINE__,
						(long) wait.count);
				break;
			}
		}

		// Initiate the respective event.
		errs	+= wait_src[index].function(wait_fd, 0);

		if (errs)
			break;		// There was an error.

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
			break;		// There was an error.

		// Wait for the resumed thread to run.
		sem_wait(&sem_exit);

		for (i = 0;; i++)
		{
			if (wait_dst.flags)
				break;

			if (i >= 100)
			{
				printf(	"FAIL <---  (%d. thread failed to resume)\n",
						__LINE__);
				errs++;
				break;
			}

			os_sleep_ms(100);
		}

		if (errs)
			break;		// There was an error.

		if (wait_src[index].wait.timeout_ms == _1_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _80_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_DONE,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _90_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
		errs++;
		break;
	}

	// Force termination, just in case of an error.
	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= sdi_ioctl(wait_fd, SDI_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;

	printf("\n");
	gsc_label_level_inc();

	// Create threads awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].wait.flags == 0; i++)
	{
		gsc_label(wait_src[i].name);
		ret	= wait_src[i].function(wait_fd, 1);

		if (ret > 0)
		{
		}
		else if (ret == 0)
		{
			printf("SKIPPED  (Test not implemented.)\n");
		}
		else
		{
			wait_fd	= fd;
			memset(&wait_dst, 0, sizeof(wait_dst));
			errs	+= _process_thread(i);
		}
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service SDI_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("SDI_IOCTL_WAIT_EVENT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	return(errs);
}


