// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_ch_grp_3_src.c $
// $Rev: 51943 $
// $Date: 2022-10-25 08:29:04 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_ch_grp_3_src
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_CH_GRP_3_SRC service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sdi_ch_grp_3_src(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			qty;
	int			ret;

	if (verbose)
		gsc_label_index("Channel Group 3 Source", index);

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_GPS, &qty);

	if (errs)
	{
		if (verbose)
			printf("FAIL <---  (querry error)\n");
	}
	else if (qty < 2)
	{
		if (verbose)
			printf("SKIPPED  (Channel Group 1 not supported.)\n");
	}
	else
	{
		ret		= sdi_ioctl(fd, SDI_IOCTL_CH_GRP_3_SRC, &set);
		errs	= ret ? 1 : 0;

		switch (set)
		{
			default:

				errs	= 1;
				ptr		= buf;
				sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
				break;

			case -1:

				ptr	= "This service is not supported by this device.";
				break;

			case SDI_CH_GRP_SRC_GEN_A:

				ptr	= "Rate Generator A";
				break;

			case SDI_CH_GRP_SRC_GEN_B:

				ptr	= "Rate Generator B";
				break;

			case SDI_CH_GRP_SRC_GEN_C:

				ptr	= "Rate Generator C";
				break;

			case SDI_CH_GRP_SRC_GEN_D:

				ptr	= "Rate Generator D";
				break;

			case SDI_CH_GRP_SRC_EXTERN:

				ptr	= "External";
				break;

			case SDI_CH_GRP_SRC_DISABLE:

				ptr	= "Disable";
				break;
		}

		if (verbose)
			printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

		if (get)
			get[0]	= set;
	}

	return(errs);
}


