// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 50362 $
// $Date: 2022-02-15 16:52:13 -0600 (Tue, 15 Feb 2022) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_query
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	sdi_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				sprintf(buf, "Query Error (#%ld)", (long) query);
				ptr	= buf;
				break;

			case SDI_QUERY_AUTO_CAL_MS:		ptr	= "Auto-Cal Period";		break;
			case SDI_QUERY_CHANNEL_D16:		ptr	= "BRR.D16 Channel Count";	break;
			case SDI_QUERY_CHANNEL_D17:		ptr	= "BRR.D17 Channel Count";	break;
			case SDI_QUERY_CHANNEL_GPS:		ptr = "Channel Groups";			break;
			case SDI_QUERY_CHANNEL_MAX:		ptr	= "Max Channels";			break;
			case SDI_QUERY_CHANNEL_QTY:		ptr	= "Input Channels";			break;
			case SDI_QUERY_COUNT:			ptr	= "Query Options";			break;
			case SDI_QUERY_DEVICE_TYPE:		ptr	= "Device Type";			break;
			case SDI_QUERY_DMDMA:			ptr = "DMDMA";					break;
			case SDI_QUERY_FGEN_MAX:		ptr = "Fgen Max";				break;
			case SDI_QUERY_FGEN_MIN:		ptr = "Fgen Min";				break;
			case SDI_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";				break;
			case SDI_QUERY_FORM_FACTOR:		ptr = "Form Factor";			break;
			case SDI_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";				break;
			case SDI_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";				break;
			case SDI_QUERY_INIT_MS:			ptr	= "Initialize Period";		break;
			case SDI_QUERY_IRQ_MASK:		ptr	= "IRQ Mask";				break;
			case SDI_QUERY_NDIV_MASK:		ptr = "Ndiv Mask";				break;
			case SDI_QUERY_NDIV_MAX:		ptr = "Ndiv Max";				break;
			case SDI_QUERY_NDIV_MIN:		ptr = "Ndiv Min";				break;
			case SDI_QUERY_NRATE_MASK:		ptr = "Nrate Mask";				break;
			case SDI_QUERY_NRATE_MAX:		ptr = "Nrate Max";				break;
			case SDI_QUERY_NRATE_MIN:		ptr = "Nrate Min";				break;
			case SDI_QUERY_RATE_GEN_QTY:	ptr = "Rate Generators";		break;
			case SDI_QUERY_THRESHOLD_MASK:	ptr = "Threshold Mask";			break;
		}

		gsc_label_index(ptr, index);
	}

	ret		= sdi_ioctl(fd, SDI_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				gsc_label_long_comma_buf(set, buf);
				break;

			case SDI_QUERY_CHANNEL_GPS:
			case SDI_QUERY_CHANNEL_MAX:
			case SDI_QUERY_CHANNEL_QTY:
			case SDI_QUERY_COUNT:
			case SDI_QUERY_NDIV_MAX:
			case SDI_QUERY_NDIV_MIN:
			case SDI_QUERY_NRATE_MAX:
			case SDI_QUERY_NRATE_MIN:
			case SDI_QUERY_RATE_GEN_QTY:

				gsc_label_long_comma_buf(set, buf);
				break;

			case SDI_QUERY_IRQ_MASK:
			case SDI_QUERY_NDIV_MASK:
			case SDI_QUERY_NRATE_MASK:
			case SDI_QUERY_THRESHOLD_MASK:

				sprintf(buf, "0x%lX", (long) set);
				break;

			case SDI_QUERY_FGEN_MAX:
			case SDI_QUERY_FGEN_MIN:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " Hz");
				break;

			case SDI_QUERY_AUTO_CAL_MS:
			case SDI_QUERY_INIT_MS:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " ms");
				break;

			case SDI_QUERY_FSAMP_MAX:
			case SDI_QUERY_FSAMP_MIN:

				gsc_label_long_comma_buf(set, buf);
				strcat(buf, " S/S");
				break;

			case SDI_QUERY_CHANNEL_D16:
			case SDI_QUERY_CHANNEL_D17:
			case SDI_QUERY_DMDMA:

				sprintf(buf, "%s", set ? "Supported" : "Not Supported");
				break;

			case SDI_QUERY_FIFO_SIZE:

				ptr	= "";
				tmp	= set;

				if ((tmp) && ((tmp % 1024) == 0))
				{
					ptr	= "K";
					tmp	/= 1024;
				}

				gsc_label_long_comma_buf(tmp, buf);
				strcat(buf, ptr);
				strcat(buf, " Words");
				break;

			case SDI_QUERY_DEVICE_TYPE:

				switch (set)
				{
					default:

						errs++;
						sprintf(buf, "INVALID: %ld", (long) set);
						break;

					case GSC_DEV_TYPE_6SDI:		strcpy(buf, "6SDI");		break;
					case GSC_DEV_TYPE_16SDI:	strcpy(buf, "16SDI");		break;
					case GSC_DEV_TYPE_16SDI_HS:	strcpy(buf, "16SDI-HS");	break;
					case GSC_DEV_TYPE_16HSDI:	strcpy(buf, "16HSDI");		break;
				}

				break;

			case SDI_QUERY_FORM_FACTOR:

				switch (set)
				{
					default:

						errs++;
						sprintf(buf, "Not Recognized: 0x%lX", (long) set);
						break;

					case SDI_FORM_FACTOR_UNKNOWN:	strcpy(buf, "UNKNOWN");	break;
					case SDI_FORM_FACTOR_PCI:		strcpy(buf, "PCI");		break;
					case SDI_FORM_FACTOR_PMC:		strcpy(buf, "PMC");		break;
					case SDI_FORM_FACTOR_CPCI:		strcpy(buf, "CPCI");	break;
					case SDI_FORM_FACTOR_PC104P:	strcpy(buf, "PC/104+");	break;
				}

				break;
		}

		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


