// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_rate_div_ndiv.c $
// $Rev: 42841 $
// $Date: 2018-05-17 18:06:37 -0500 (Thu, 17 May 2018) $

// 16SDI: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	sdi_rate_div_ndiv_all
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_RATE_DIV_XX_NDIV service
*		for all channels.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		The setting to apply.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int	sdi_rate_div_ndiv_all(int fd, int index, int verbose, s32 set)
{
	int	errs;
	s32	qty		= 0;

	if (verbose)
		gsc_label_index("Rate Divisors", index);

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &qty);

	if (verbose)
		printf("%s  (%ld)\n", errs ? "FAIL <---" : "PASS", (long) qty);

	gsc_label_level_inc();

	if (qty > 0)
		errs	+= sdi_rate_div_00_ndiv(fd, index, verbose, set, NULL);

	if (qty > 1)
		errs	+= sdi_rate_div_01_ndiv(fd, index, verbose, set, NULL);

	if (qty > 2)
		errs	+= sdi_rate_div_02_ndiv(fd, index, verbose, set, NULL);

	if (qty > 3)
		errs	+= sdi_rate_div_03_ndiv(fd, index, verbose, set, NULL);

	if (qty > 4)
		errs	+= sdi_rate_div_04_ndiv(fd, index, verbose, set, NULL);

	if (qty > 5)
		errs	+= sdi_rate_div_05_ndiv(fd, index, verbose, set, NULL);

	if (qty > 6)
		errs	+= sdi_rate_div_06_ndiv(fd, index, verbose, set, NULL);

	if (qty > 7)
		errs	+= sdi_rate_div_07_ndiv(fd, index, verbose, set, NULL);

	if (qty > 8)
		errs	+= sdi_rate_div_08_ndiv(fd, index, verbose, set, NULL);

	if (qty > 9)
		errs	+= sdi_rate_div_09_ndiv(fd, index, verbose, set, NULL);

	if (qty > 10)
		errs	+= sdi_rate_div_10_ndiv(fd, index, verbose, set, NULL);

	if (qty > 11)
		errs	+= sdi_rate_div_11_ndiv(fd, index, verbose, set, NULL);

	if (qty > 12)
		errs	+= sdi_rate_div_12_ndiv(fd, index, verbose, set, NULL);

	if (qty > 13)
		errs	+= sdi_rate_div_13_ndiv(fd, index, verbose, set, NULL);

	if (qty > 14)
		errs	+= sdi_rate_div_14_ndiv(fd, index, verbose, set, NULL);

	if (qty > 15)
		errs	+= sdi_rate_div_15_ndiv(fd, index, verbose, set, NULL);

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	sdi_rate_div_ndiv_chan
*
*	Purpose:
*
*		Provide a visual wrapper for the SDI_IOCTL_RATE_DIV_XX_NDIV service
*		for a specific channel.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		chan	This is the channel for the rate dividor of interest. Use -1 for all.
*
*		set		The setting to apply.
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int sdi_rate_div_ndiv_chan(int fd, int index, int verbose, int chan, s32 set, s32* get)
{
	int	errs;
	s32	qty;

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &qty);

	if ((errs) && (verbose))
		printf(" FAIL <---  (sdi_query() failure)\n");

	if ((chan + 1) <= qty)
	{
		switch (chan)
		{
			case 0:		errs	+= sdi_rate_div_00_ndiv(fd, index, verbose, set, get);	break;
			case 1:		errs	+= sdi_rate_div_01_ndiv(fd, index, verbose, set, get);	break;
			case 2:		errs	+= sdi_rate_div_02_ndiv(fd, index, verbose, set, get);	break;
			case 3:		errs	+= sdi_rate_div_03_ndiv(fd, index, verbose, set, get);	break;
			case 4:		errs	+= sdi_rate_div_04_ndiv(fd, index, verbose, set, get);	break;
			case 5:		errs	+= sdi_rate_div_05_ndiv(fd, index, verbose, set, get);	break;
			case 6:		errs	+= sdi_rate_div_06_ndiv(fd, index, verbose, set, get);	break;
			case 7:		errs	+= sdi_rate_div_07_ndiv(fd, index, verbose, set, get);	break;
			case 8:		errs	+= sdi_rate_div_08_ndiv(fd, index, verbose, set, get);	break;
			case 9:		errs	+= sdi_rate_div_09_ndiv(fd, index, verbose, set, get);	break;
			case 10:	errs	+= sdi_rate_div_10_ndiv(fd, index, verbose, set, get);	break;
			case 11:	errs	+= sdi_rate_div_11_ndiv(fd, index, verbose, set, get);	break;
			case 12:	errs	+= sdi_rate_div_12_ndiv(fd, index, verbose, set, get);	break;
			case 13:	errs	+= sdi_rate_div_13_ndiv(fd, index, verbose, set, get);	break;
			case 14:	errs	+= sdi_rate_div_14_ndiv(fd, index, verbose, set, get);	break;
			case 15:	errs	+= sdi_rate_div_15_ndiv(fd, index, verbose, set, get);	break;
		}
	}

	return(errs);
}


