// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16SDI/16SDI_Linux_4.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 51943 $
// $Date: 2022-10-25 08:29:04 -0500 (Tue, 25 Oct 2022) $

// 16SDI: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)		(sizeof((a)) / sizeof((a)[0]))

#define	_GSC_REG(a)	"GSC " #a, SDI_GSC_##a, 0, 0

#define	SHOW_FIELD(begin,end,eol,list,name)	gsc_reg_field_show(label_w + 10, 23, value,(begin),(end),(eol),(list),(name))



//*****************************************************************************
static int _bcr_decode(int fd, int supported, u32 value, int label_w)
{
	static const char*	ain[]		= {	"Differential",
										"Single-Ended",
										"Zero Test",
										"Vref Test" };

	static const char*	auto_cal[]	= {	"Inactive",
										"In Progress" };

	static const char*	autocl_ps[]	= {	"FAIL <---",
										"Pass" };

	static const char*	ready[]		= {	"Busy",
										"Ready" };

	static const char*	init[]		= {	"Done",
										"Busy" };

	static const char*	initiate[]	= {	"Target",
										"Initiator" };

	static const char*	irq_list[]	= {	"Initialization Complete",
										"Auto Cal Done",
										"Channels Ready",
										"Buffer Threshold LOW-to-HIGH",
										"Buffer Threshold HIGH-to-LOW",
										"Xfer Buffer Almost Empty",
										"Xfer Buffer Almost Full",
										"RESERVED" };

	static const char*	irq_flag[]	= {	"Negated",
										"Asserted" };

	static const char*	th_flag[]	= {	"Negated",
										"Asserted" };

	static const char*	offset[]	= {	"Twos Compliment",
										"Offset Binary" };

	static const char*	on_sync[]	= {	"Synchronize",
										"Clear Buffer" };

	static const char*	range[]		= {	"+-1.25V",
										"+-2.5V",
										"+-5V",
										"+-10V" };

	static const char*	sw_sync[]	= {	"Negated",
										"Asserted" };

	static const char*	sync_scan[]	= {	"Asynchronous",
										"Synchronous" };

	SHOW_FIELD(31, 18, 1, NULL,			"Reserved"				);
	SHOW_FIELD(17, 17, 1, on_sync,		"Clear Buffer On Sync"	);
	SHOW_FIELD(16, 16, 1, sync_scan,	"Synchronize Scan"		);
	SHOW_FIELD(15, 15, 1, init,			"Initialize"			);
	SHOW_FIELD(14, 14, 1, th_flag,		"Buffer Threshold Flag"	);
	SHOW_FIELD(13, 13, 1, ready,		"Channels Ready"		);
	SHOW_FIELD(12, 12, 1, autocl_ps,	"Auto Cal Pass"			);
	SHOW_FIELD(11, 11, 1, irq_flag,		"IRQ Request Flag"		);
	SHOW_FIELD(10,  8, 1, irq_list,		"Interrupt"				);
	SHOW_FIELD( 7,  7, 1, auto_cal,		"Auto Cal"				);
	SHOW_FIELD( 6,  6, 1, sw_sync,		"Software Sync"			);
	SHOW_FIELD( 5,  5, 1, initiate,		"Initiator"				);
	SHOW_FIELD( 4,  4, 1, offset,		"Offset Binary"			);
	SHOW_FIELD( 3,  2, 1, range,		"Range"					);
	SHOW_FIELD( 1,  0, 1, ain,			"Analog Input Mode"		);
	return(supported);
}



//*****************************************************************************
static int _brr_decode(int fd, int supported, u32 value, int label_w)
{
	static const char*	dma[]	= {	"Block Mode DMA Only",
									"BMDMA and DMDMA are supported" };

	char	buf[32];
	s32		chans;
	s32		d16;
	s32		d17;
	int		errs	= 0;
	u32		field;

	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &chans);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_D16, &d16);
	errs	+= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_D17, &d17);

	SHOW_FIELD(31, 18, 1, NULL, "Reserved");

	// The D17 bit.

	if (d17 < 0)
	{
		SHOW_FIELD(17, 17, 1, NULL, "Reserved");
	}
	else
	{
		sprintf(buf, "%d Channels", d17);
		SHOW_FIELD(17, 17, 0, NULL, buf);

		if (value & (1 << 17))
			printf("Yes, this device has %d channels.\n", (int) d17);
		else
			printf("No, this device has %d channels.\n", (int) chans);
	}

	// The D16 bit.

	if (d16 < 0)
	{
		SHOW_FIELD(16, 16, 1, NULL, "Reserved");
	}
	else
	{
		sprintf(buf, "%d Channels", d16);
		SHOW_FIELD(16, 16, 0, NULL, buf);

		if (value & (1 << 16))
			printf("Yes, this device has %d channels.\n", (int) d16);
		else
			printf("No, this device has %d channels.\n", (int) chans);
	}

	// other field
	SHOW_FIELD(15, 15, 1, dma,	"DMA Support"		);
	SHOW_FIELD(14, 12, 1, NULL, "Reserved"			);
	SHOW_FIELD(11,  0, 0, NULL, "Firmware Revision"	);
	field	= GSC_FIELD_DECODE(value, 11, 0);
	printf("%03lX\n", (long) field);

	return(errs);
}



//*****************************************************************************
static int _bsr_decode(int fd, int supported, u32 value, int label_w)
{
	int	errs;
	int	hi_bit;
	u32	field;
	s32	mask;

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_THRESHOLD_MASK, &mask);

	if (errs == 0)
	{
		for (hi_bit = 31; hi_bit; hi_bit--)
		{
			if (mask & (1L << hi_bit))
				break;
		}

		SHOW_FIELD(31, hi_bit + 1,	1, NULL, "Reserved"	);
		SHOW_FIELD(hi_bit, 0,		0, NULL, "Samples"	);
		field	= GSC_FIELD_DECODE(value, hi_bit, 0);
		printf("%ld Samples\n", (long) field);
	}

	return(supported);
}



//*****************************************************************************
static int _btr_decode(int fd, int supported, u32 value, int label_w)
{
	static const char*	clear[]	= {	"Don't Clear",
									"Clear" };

	static const char*	input[]	= {	"Enable",
									"Disable" };

	int	errs;
	s32	mask;

	SHOW_FIELD(31, 20, 1, NULL,		"Reserved"				);
	SHOW_FIELD(19, 19, 1, clear,	"Clear Buffer"			);
	SHOW_FIELD(18, 18, 1, input,	"Disable Buffer Input"	);

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_THRESHOLD_MASK, &mask);

	if (errs)
	{
	}
	else if (mask & 0x20000)
	{
		SHOW_FIELD(17,  0, 1, NULL, "Threshold"	);
	}
	else if (mask & 0x10000)
	{
		SHOW_FIELD(17, 17, 1, NULL, "Reserved"	);
		SHOW_FIELD(16,  0, 1, NULL, "Threshold"	);
	}
	else
	{
		SHOW_FIELD(17, 16, 1, NULL, "Reserved"	);
		SHOW_FIELD(15,  0, 1, NULL, "Threshold"	);
	}

	return(supported);
}



//*****************************************************************************
static int _idbr_decode(int fd, int supported, u32 value, int label_w)
{
	u32	field;

	SHOW_FIELD(31, 19,	1, NULL, "Reserved"		);
	SHOW_FIELD(18, 16,	1, NULL, "Channel Tag"	);
	SHOW_FIELD(15,  0,	0, NULL, "ADC Value"	);
	field	= GSC_FIELD_DECODE(value, 15, 0);
	printf("0x%04lX\n", (long) field);

	return(supported);
}



//*****************************************************************************
static int _rar_decode(int fd, int supported, u32 value, int label_w)
{
	static const char*	assign[]	= {	"Rate Generator A",
										"Rate Generator B",
										"Rate Generator C",
										"Rate Generator D",
										"External Sample Clock",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable",
										"Disable" };

	int	errs;
	s32	groups;

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_GPS, &groups);

	if (errs == 0)
	{
		SHOW_FIELD(31, 16, 1, NULL, "Reserved");

		if (groups > 3)
			SHOW_FIELD(15, 12, 1, assign,	"Assign 3");
		else
			SHOW_FIELD(15, 12, 1, NULL,		"Reserved");

		if (groups > 2)
			SHOW_FIELD(11,  8, 1, assign,	"Assign 2");
		else
			SHOW_FIELD(11,  8, 1, NULL,		"Reserved");

		SHOW_FIELD(7, 4, 1,	assign, "Assign 1");
		SHOW_FIELD(3, 0, 1,	assign, "Assign 0");
	}

	return(supported);
}



//*****************************************************************************
static int _rcxr_decode(int fd, int index, int supported, u32 value, int label_w)
{
	int		errs	= 0;
	float	fgen;
	s32		model;
	u32		nrate;
	s32		nrate_mask;
	s32		qty;

	if (supported)
	{
		errs	= sdi_query(fd, -1, 0, SDI_QUERY_RATE_GEN_QTY, &qty);

		if (errs)
			supported	= 0;
		else if (index < qty)
			supported	= 1;
		else
			supported	= 0;
	}
	else
	{
		SHOW_FIELD(31, 9, 1,	NULL, "Reserved");

		// Show Nrate and its results.
		SHOW_FIELD(8, 0, 0,		NULL, "Nrate"	);
		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_NRATE_MASK, &nrate_mask);
		errs	+= sdi_query(fd, -1, 0, SDI_QUERY_DEVICE_TYPE, &model);
		nrate	= value & nrate_mask;

		if (errs == 0)
		{
			switch (model)
			{
				default:

						errs	= 1;
						printf(	"FAIL <---  (%d. INTERNAL ERROR: model %ld)\n",
								__LINE__,
								(long) model);
						break;

					case GSC_DEV_TYPE_6SDI:
					case GSC_DEV_TYPE_16SDI:

						fgen	= SDI_FGEN_BASIC_KHZ(nrate);
						printf("%-3ld  (Fgen = %.4f KHz)\n", (long) nrate, (float) fgen);
						break;

					case GSC_DEV_TYPE_16SDI_HS:
					case GSC_DEV_TYPE_16HSDI:

						fgen	= SDI_FGEN_HS_MHZ(nrate);
						printf("%-3ld  (Fgen = %.4f MHz)\n", (long) nrate, (float) fgen);
						break;
			}
		}

		supported	= errs;
	}

	return(supported);
}



//*****************************************************************************
static int _rcar_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rcxr_decode(fd, 0, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rcbr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rcxr_decode(fd, 1, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rccr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rcxr_decode(fd, 2, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rcdr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rcxr_decode(fd, 3, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static void _rdxr_decode_field(int fd, int index, u32 value, int label_w, int hi_bit, int low_bit)
{
	s32		model;
	int		errs;
	u32		field;
	int		hs;			// High Speed model?
	char	name[64];

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_DEVICE_TYPE, &model);

	if (errs == 0)
	{
		switch (model)
		{
			default:
			case GSC_DEV_TYPE_6SDI:
			case GSC_DEV_TYPE_16SDI:	hs	= 0;	break;
			case GSC_DEV_TYPE_16SDI_HS:
			case GSC_DEV_TYPE_16HSDI:	hs	= 1;	break;
		}

		sprintf(name, "Chan %02d Ndiv", index);
		field	= GSC_FIELD_DECODE(value, hi_bit, low_bit);

		if (hs == 0)
		{
			SHOW_FIELD(hi_bit, low_bit, 1, NULL, name);
		}
		else if (field)
		{
			SHOW_FIELD(hi_bit, low_bit, 0, NULL, name);
			printf("%d  (DIVISOR = %d)\n", (int) field, (int) field);
		}
		else
		{
			SHOW_FIELD(hi_bit, low_bit, 0, NULL, name);
			printf("%d  (DIVISOR = 0.5)\n", (int) field);
		}
	}
}



//*****************************************************************************
static int _rdxr_decode(int fd, int index, int supported, u32 value, int label_w)
{
	s32	chans;
	int	errs;

	errs	= sdi_query(fd, -1, 0, SDI_QUERY_CHANNEL_QTY, &chans);

	if (supported)
	{
		if (errs)
			supported	= 0;
		else if (index >= chans)
			supported	= 0;
		else
			supported	= 1;
	}
	else if (errs)
	{
	}
	else
	{
		SHOW_FIELD(31, 14, 1, NULL, "Reserved");
		_rdxr_decode_field(fd, index + 1, value, label_w, 13, 8);
		SHOW_FIELD( 7,  6, 1, NULL, "Reserved");
		_rdxr_decode_field(fd, index, value, label_w, 5, 0);
	}

	return(supported);
}



//*****************************************************************************
static int _rd01r_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 0, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rd23r_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 2, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rd45r_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 4, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rd67r_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 6, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rd89r_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 8, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rdabr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 10, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rdcdr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 12, supported, value, label_w);
	return(supported);
}



//*****************************************************************************
static int _rdefr_decode(int fd, int supported, u32 value, int label_w)
{
	supported	= _rdxr_decode(fd, 14, supported, value, label_w);
	return(supported);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCR),	0,	_bcr_decode,	"Board Control Register"		},
	{ _GSC_REG(RCAR),	1,	_rcar_decode,	"Rate Control A Register"		},
	{ _GSC_REG(RCBR),	1,	_rcbr_decode,	"Rate Control B Register"		},
	{ _GSC_REG(RCCR),	1,	_rccr_decode,	"Rate Control C Register"		},
	{ _GSC_REG(RCDR),	1,	_rcdr_decode,	"Rate Control D Register"		},
	{ _GSC_REG(RAR),	0,	_rar_decode,	"Rate Assignments Register"		},
	{ _GSC_REG(RD01R),	1,	_rd01r_decode,	"Rate Divisor 00,01 Register"	},
	{ _GSC_REG(RD23R),	1,	_rd23r_decode,	"Rate Divisor 02,03 Register"	},
	{ _GSC_REG(RD45R),	1,	_rd45r_decode,	"Rate Divisor 04,05 Register"	},
	{ _GSC_REG(RD67R),	1,	_rd67r_decode,	"Rate Divisor 06,07 Register"	},
	{ _GSC_REG(RD89R),	1,	_rd89r_decode,	"Rate Divisor 08,09 Register"	},
	{ _GSC_REG(RDABR),	1,	_rdabr_decode,	"Rate Divisor 10,11 Register"	},
	{ _GSC_REG(RDCDR),	1,	_rdcdr_decode,	"Rate Divisor 12,13 Register"	},
	{ _GSC_REG(RDEFR),	1,	_rdefr_decode,	"Rate Divisor 14,15 Register"	},
	{ _GSC_REG(BTR),	0,	_btr_decode,	"Buffer Threshold Register"		},
	{ _GSC_REG(BRR),	0,	_brr_decode,	"Board Revision Register"		},
	{ _GSC_REG(BSR),	0,	_bsr_decode,	"Buffer Size Register"			},
	{ _GSC_REG(ACVR),	0,	NULL,			"Auto Cal Values Register"		},
	{ _GSC_REG(IDBR),	0,	_idbr_decode,	"Input Data Buffer Register"	},
	{ NULL, 0, 0, 0,	0,	NULL,			NULL							}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	sdi_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* sdi_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	sdi_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* sdi_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	sdi_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* sdi_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	sdi_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* sdi_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	sdi_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int sdi_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, sdi_reg_read);
	return(errs);
}


